import { useRetrieve } from '../use-retrieve'
import { unref, ref } from 'vue'
import { ElMessage } from 'element-plus'

export const useFormDialog = ({ beforeOpen = () => {}, submit = () => {} } = {}) => {
  const dialog = ref(false)
  const formItem = ref({})
  const openDialog = (params, type) => {
    beforeOpen(params, type)
    dialog.value = true
  }
  const onConfirm = async (resolve, reject) => {
    try {
      await submit()
      resolve()
    } catch (e) {
      reject(e)
    }
  }
  return {
    dialog, openDialog, formItem, onConfirm
  }
}

export const useCreate = (Entity, { itemType, pageFn, infoFn, createFn = 'create' }) => {
  const Retrieve = useRetrieve(Entity, { itemType, pageFn, infoFn })
  const { item, dialogType, showOnly, itemDialog } = Retrieve
  const createItem = (initValue = {}) => {
    item.value = initValue
    dialogType.value = '新增'
    showOnly.value = false
    itemDialog.value = true
  }
  const saveItem = async (resolve, reject) => {
    try {
      const method = unref(createFn)
      const res = await unref(Entity)[method](item.value)
      ElMessage({ type: 'success', message: res.message })
      // eslint-disable-next-line no-unused-expressions
      resolve?.(res)
      return res
    } catch (e) {
      // eslint-disable-next-line no-unused-expressions
      reject?.(e)
      throw e
    }
  }
  return {
    ...Retrieve,
    createItem,
    saveItem
  }
}
