import { useRetrieve } from '../use-retrieve'
import { cloneDeep } from '@cip/utils/util'
import { unref, ref } from 'vue'
import { ElMessage, ElMessageBox } from 'element-plus'

export const useCurd = (Entity, {
  itemType = '',
  pageFn,
  infoFn,
  createFn = 'create',
  updateFn = 'update',
  deleteFn = 'delete',
  batchDeleteFn = 'batchDelete',
  judgeUpdate
}) => {
  const Retrieve = useRetrieve(Entity, { itemType, pageFn, infoFn })
  const { item, dialogType, showOnly, itemDialog, getItem, getItemList } = Retrieve
  const isUpdate = ref(false)
  if (!judgeUpdate) {
    judgeUpdate = () => item.value.id
  }
  const createItem = (initValue = {}, type = '新增') => {
    isUpdate.value = false // 设置更新标记
    item.value = { ...initValue }
    dialogType.value = type
    showOnly.value = false
    itemDialog.value = true
  }
  /**
   * @deprecated
   */
  const editItem = (...args) => {
    updateItem(...args)
  }
  const updateItem = async (info, fetch = false, type = '编辑') => {
    isUpdate.value = true // 设置更新标记
    try {
      if (fetch) {
        // 接口请求时间过长将导致页面卡顿且无提示，故加入_loading
        info._updateLoading = true
        try {
          const { data } = await getItem(info)
          item.value = data
          // eslint-disable-next-line no-useless-catch
        } catch (e) {
          // 抛出错误
          throw e
        } finally {
          info._updateLoading = false
        }
      } else {
        item.value = cloneDeep(info)
      }
      dialogType.value = type
      showOnly.value = false
      itemDialog.value = true
    } catch (e) {

    }
  }
  // 保存修改或新建
  const saveItem = async (resolve, reject) => {
    try {
      const method = judgeUpdate() ? unref(updateFn) : unref(createFn)
      const res = await unref(Entity)[method](item.value)
      ElMessage({ type: 'success', message: res.message })
      getItemList()
      // eslint-disable-next-line no-unused-expressions
      resolve?.(res)
      return res
    } catch (e) {
      // eslint-disable-next-line no-unused-expressions
      reject?.(e)
    }
  }

  const deleteItem = async (info, message, rewrite = false) => {
    const confirmMessage = rewrite ? message : `确认删除 ${message || ''}`
    try {
      await ElMessageBox.confirm(confirmMessage, '提示', { type: 'warning' })
      try {
        const { message } = await unref(Entity)[unref(deleteFn)](info)
        ElMessage({ type: 'success', message })
        getItemList()
      } catch (e) {
        console.error(e)
      }
    } catch (e) {
      console.info('取消删除')
    }
  }
  const deleteItemList = async (beforeDelete) => {
    // 此处itemList为内部变量非Retrieve中的itemList
    const itemList = unref(beforeDelete?.()) || []
    if (!itemList || itemList.length === 0) {
      ElMessage.error(`需要选择至少一条${unref(itemType) ?? '数据'}后再进行批量删除`)
      return
    }
    try {
      await ElMessageBox.confirm('确认批量删除已选项', '提示', { type: 'warning' })
      try {
        const { message } = await unref(Entity)[unref(batchDeleteFn)](itemList)
        ElMessage.success({ type: 'success', message })
        getItemList()
      } catch (e) {
        console.error(e)
      }
    } catch (e) {
      console.info('取消批量删除')
    }
  }
  return {
    ...Retrieve,
    isUpdate,
    createItem,
    editItem,
    updateItem,
    saveItem,
    deleteItem,
    deleteItemList
  }
}
